@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013-2014 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2013-2017 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -175,7 +175,7 @@ namespace mrn {
 
     bool convertable = false;
 
-    enum_field_types field_type = field_item->field_type();
+    enum_field_types field_type = field_item->field->real_type();
     NormalizedType normalized_type = normalize_field_type(field_type);
     switch (normalized_type) {
     case STRING_TYPE:
@@ -185,7 +185,12 @@ namespace mrn {
       }
       break;
     case INT_TYPE:
-      convertable = value_item->type() == Item::INT_ITEM;
+      if (field_type == MYSQL_TYPE_ENUM) {
+        convertable = (value_item->type() == Item::STRING_ITEM ||
+                       value_item->type() == Item::INT_ITEM);
+      } else {
+        convertable = value_item->type() == Item::INT_ITEM;
+      }
       break;
     case TIME_TYPE:
       if (is_valid_time_value(field_item, value_item)) {
@@ -564,7 +569,7 @@ namespace mrn {
                                              grn_obj *expression) {
     MRN_DBUG_ENTER_METHOD();
 
-    enum_field_types field_type = field_item->field_type();
+    enum_field_types field_type = field_item->field->real_type();
     NormalizedType normalized_type = normalize_field_type(field_type);
 
     switch (normalized_type) {
@@ -578,7 +583,21 @@ namespace mrn {
       break;
     case INT_TYPE:
       grn_obj_reinit(ctx_, &value_, GRN_DB_INT64, 0);
-      GRN_INT64_SET(ctx_, &value_, const_item->val_int());
+      if (field_type == MYSQL_TYPE_ENUM) {
+        if (const_item->type() == Item::STRING_ITEM) {
+          String *string;
+          string = const_item->val_str(NULL);
+          Field_enum *enum_field = static_cast<Field_enum *>(field_item->field);
+          int enum_value = find_type(string->c_ptr(),
+                                     enum_field->typelib,
+                                     FIND_TYPE_BASIC);
+          GRN_INT64_SET(ctx_, &value_, enum_value);
+        } else {
+          GRN_INT64_SET(ctx_, &value_, const_item->val_int());
+        }
+      } else {
+        GRN_INT64_SET(ctx_, &value_, const_item->val_int());
+      }
       break;
     case TIME_TYPE:
       grn_obj_reinit(ctx_, &value_, GRN_DB_TIME, 0);
