@@ -14,7 +14,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef MRN_MYSQL_H_
@@ -43,27 +43,16 @@
 #define MYSQL_SERVER 1
 #include <mysql_version.h>
 
-#if MYSQL_VERSION_ID < 50500
-#  include <mysql_priv.h>
-#  include <mysql/plugin.h>
-#else
-#  include <sql_priv.h>
-#  include <sql_class.h>
-#  include <probes_mysql.h>
-#  include <sql_partition.h>
-#endif
+#include <sql_const.h>
+#include <sql_class.h>
+#include <probes_mysql.h>
+#include <sql_partition.h>
 #include <rpl_filter.h>
 
 #ifdef MARIADB_BASE_VERSION
 #  define MRN_MARIADB_P 1
 #endif
 
-#if MYSQL_VERSION_ID >= 50607
-#  if !defined(MRN_MARIADB_P)
-#    define MRN_HAVE_SQL_OPTIMIZER_H
-#  endif
-#endif
-
 #define MRN_MESSAGE_BUFFER_SIZE 1024
 
 #define MRN_DBUG_ENTER_FUNCTION() DBUG_ENTER(__FUNCTION__)
